from PyQt4.QtCore import *
from PyQt4.QtGui import *

from dockWidget_ui import Ui_DockWidget
from toolSelect import *
from algorithm import *

class DockWidget (QDockWidget, Ui_DockWidget):
    def __init__(self, iface, parent, canvas, graph):
        QDockWidget.__init__(self, parent)
        self.setupUi( self )
        self.setAllowedAreas( Qt.LeftDockWidgetArea | Qt.RightDockWidgetArea )
        self.setContentsMargins ( 0, 0, 0, 0 )

        self.iface = iface
        self.canvas = canvas
        self.graph = graph
        self.starting_dipole = ""
        self.target_dipole = ""

        self.connect(self.btnStartingDipole, SIGNAL("clicked()"), self.selectStartingDipole)
        self.connect(self.btnTargetDipole, SIGNAL("clicked()"), self.selectTargetDipole)
        self.connect(self.btnCalculateRoute, SIGNAL("clicked()"), self.calculateRoute)
        self.connect(self.btnClear, SIGNAL("clicked()"), self.clear)

    def selectStartingDipole(self):
        self.toolSelectStartingDipole = SeleccionarMapTool( self.canvas, True )
        self.canvas.setMapTool( self.toolSelectStartingDipole )
        
        self.connect(self.toolSelectStartingDipole, SIGNAL("startingFeatureSelected"), self.startingFeatureSelected)

    def selectTargetDipole(self):
        self.toolSelectTargetDipole = SeleccionarMapTool( self.canvas, False )
        self.canvas.setMapTool( self.toolSelectTargetDipole )

        self.connect(self.toolSelectTargetDipole, SIGNAL("targetFeatureSelected"), self.targetFeatureSelected)
 
    def startingFeatureSelected(self, fid):
        self.starting_dipole = self.graph.getEdgeIdFromFeatId(fid)
        self.lblStartingDipole.setText("id:" + str(fid) + ", dipole:" + str(self.starting_dipole))

    def targetFeatureSelected(self, fid):
        self.target_dipole = self.graph.getEdgeIdFromFeatId(fid)
        self.lblTargetDipole.setText("id:" + str(fid) + ", dipole:" + str(self.target_dipole))

    def clear(self):
        #for i in self.canvas.scene().items():
        #    self.canvas.scene().removeItem(i)
        self.lblStartingDipole.setText("Not yet selected")
        self.lblTargetDipole.setText("Not yet selected")
        self.txtLog.setText("")
        self.starting_dipole = ""
        self.target_dipole = ""

    def calculateRoute(self):
    
        if self.starting_dipole == "" or self.target_dipole == "":
            QMessageBox.warning( self.canvas, "Warning", 
                    "Please select the starting and target dipoles" )
            return

        algorithm = Algorithm( self.graph )
        route = algorithm.run( self.starting_dipole, self.target_dipole )

        if route: # Create memory layer and add it to QGIS

            for t in algorithm.getMessageToPrint():
                self.txtLog.append(t)

            vl = QgsVectorLayer("LineString", "Calculated Route", "memory")
            pr = vl.dataProvider()
            pr.addAttributes( [ ] ) 

            # Repeated code from qualitativeroute.py for getting the layer 0
            legend = self.iface.legendInterface() 
            if len(legend.layers()) == 0:
                QMessageBox.warning( self.iface.mainWindow(), "Warning", 
                        "Please load a layer to the map" )
                return
            layer = legend.layers()[0]
            if not layer.type() == 0: # Not a vector layer
                QMessageBox.warning( self.iface.mainWindow(), "Warning", 
                    "The first layer has to be a vector layer with lines" )
                return

            # add features
            for edge in route:
                current_fid = self.graph.getFeatIdFromEdgeId( edge )

                if current_fid:
                    feat = QgsFeature()
                    layer.featureAtId(current_fid, feat, True, True)
                    #feat.setGeometry( QgsGeometry.fromPolyline( points ) )  
                    pr.addFeatures( [ feat ] )

            vl.updateExtents()
            QgsMapLayerRegistry.instance().addMapLayer( vl )



